const assert = require('assert');
const axios = require('axios');
const grpc = require('@grpc/grpc-js');
const http = require('http');

const { ProjectorService } = require('./generated/projector_grpc_pb');
const { Point } = require('./generated/projector_pb');
const database = require('./database');

const port = process.env.PORT || 9090;
const projectorUrl = process.env.PROJECTOR_URL || 'http://localhost:8090';

//OLD
function sendPoint(call, callback) {
  axios.get(`${projectorUrl}/generatePoint`).then(resp=>{
    callback(null, resp["data"]["point"]);
  })
}

//Create the query and execute it
function addToDB(call){

  //TEMP disable
  if(true){
    return;
  }


  
  data = call.request.rows;
  var query = 'INSERT INTO SAMPLE.dbo.pointTable (x,y) \n VALUES \n'

  for(var i = 0; i < data.length-1; i++){
    hdvector = data[i].hdvector;
    query += `(${hdvector[1]}, ${hdvector[2]}),\n`
  }
  query += `(${data[data.length-1].hdvector[1]}, ${data[data.length-1].hdvector[2]});`
  database.executeQuery(query);
}

//Route the grpc function to the microservice
function sendPointStream(call) {
  
  var modelID = "undefined";
  if(call.request.modelid){
    //Non-empty training set
    addToDB(call);
    modelID = call.request.modelid;
  }

  var requestURL = `${projectorUrl}/generatePointStream?modelID=${modelID}`;
    
   http.get(requestURL, res =>{

    res.on("data", data =>{
      sendDataStream(data,call)
    })

    res.on("end",function() {
        call.end();
    })
  })  
}

//Right now it doesnt fix cut off jsons (FIX LATER)
function sendDataStream(data,call){
  var list = data.toString().trim().split("\n")

  for(var i =0; i <list.length;i++){
    try {
      var response = JSON.parse(list[i]);
      var point = new Point();
      point.setId(response["point"]["id"]);
      point.setX(response["point"]["x"]);
      point.setY(response["point"]["y"]);
      call.write(point);
    } catch (e) {}
  }
}

//Create server and route the grpc functions to js functions
function getServer() {

  var server = new grpc.Server();
  server.addService(ProjectorService, {
    getProjectionPoints: sendPointStream,
  });
  console.log("started");
  return server;
}

//Setup
if (require.main === module) {
  var server = getServer();
  server.bindAsync(
    `0.0.0.0:${port}`, grpc.ServerCredentials.createInsecure(), (err, port) => {
      assert.ifError(err);
      server.start();
      console.log("connected");
      console.log(`Server listening at http://0.0.0.0:${port}`);
  });
}
exports.getServer = getServer;
